package net.java.dev.esjug.util.persistence;

import java.util.Collection;

import org.springframework.orm.hibernate3.support.HibernateDaoSupport;


/**
 * Base class for DAOs that use Hibernate's Spring integration
 * as persistence mechanism.
 *
 * </p><p>
 * For more information on the DAO pattern, visit:
 * <a href="http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html" target="_blank">http://java.sun.com/blueprints/corej2eepatterns/Patterns/DataAccessObject.html</a>.
 * 
 * @see net.java.dev.esjug.util.persistence.BaseDAO
 * @author Vitor Souza (vitorsouza@gmail.com)
 */
public abstract class HibernateSpringBaseDAO<T> extends HibernateDaoSupport implements BaseDAO<T> {
	/**
	 * Abstract method to be implemented by subclasses in order to return
	 * the domain class that is persisted by each DAO.
	 * 
	 * @return Domain class that is persisted by DAO subclasses.
	 */
	protected abstract Class getDomainClass();
	
	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.persistence.DAOBase#recuperarTodos()
	 */
	@SuppressWarnings("unchecked")
	public Collection<T> retrieveAll() {
		// Uses Spring's Hibernate support to load all objects from a class.
		return getHibernateTemplate().loadAll(getDomainClass());
	}

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.persistence.DAOBase#recuperarPorId(java.lang.Long)
	 */
	@SuppressWarnings("unchecked")
	public T retrieveById(Long id) {
		// Uses Spring's Hibernate support to retrieve an object given its id.
		return (T)getHibernateTemplate().load(getDomainClass(), id);
	}

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.persistence.DAOBase#salvar(java.lang.Object)
	 */
	public void save(T object) {
		// Uses Spring's Hibernate support to save an object.
		getHibernateTemplate().saveOrUpdate(object);
	}

	/* (non-Javadoc)
	 * @see net.java.dev.esjug.util.persistence.DAOBase#excluir(java.lang.Object)
	 */
	public void delete(T object) {
		// Uses Spring's Hibernate support to delete an object.
		getHibernateTemplate().delete(object);
	}
}
